//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TDirCreator.h"
#include "SystemUtils.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// class EDirCreationFail : public Exception
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall EDirCreationFail::EDirCreationFail(AnsiString AMessage) :
	Exception(AMessage)
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
namespace
{
const char* EXCEPTION_CANNOT_CREATE_DIR =
	"Cannot create the directory \"%s\".";
const char* EXCEPTION_CANNOT_GET_LOCATION =
	"Cannot get the location to create directories.";
const char* EXCEPTION_CANNOT_MAKE_DIR_NAME =
	"Cannot make a directory name with \"%s\".";
const char* EXCEPTION_INVALID_SRC_NAME =
	"\"%s\" is an invalid source name.";
const char* EXCEPTION_UNSUPPORTED_CONDITION =
	"An unsupported condition to create directories.";
} // End of namespace
//---------------------------------------------------------------------------
// class TDirCreator : public TObject
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall TDirCreator::Initialize(TStrings* ASrcNames)
{
	try {
		FSrcNames = new TStringList();
		SrcNames->Assign(ASrcNames);
		Done = 0;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TDirCreator::Uninitialize()
{
	delete FSrcNames;
}
//---------------------------------------------------------------------------
void __fastcall TDirCreator::CreateNextDir()
{
	bool __fastcall (*createDir)(AnsiString) = Options.CreateLocation ?
		ForceDirectories :
		CreateDir;
	AnsiString dirName = GetNextDirName();
	if(!createDir(dirName)) {
		throw EDirCreationFail(AnsiString().sprintf(
			EXCEPTION_CANNOT_CREATE_DIR, dirName.c_str()));
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDirCreator::GetNextDirName()
{
	AnsiString srcName = GetNextSrcName();
	AnsiString path = GetNextPath(srcName);
	int index = Options.FirstIndex - 1;
	while(++index <= Options.LastIndex) {
		// Get name part
		AnsiString namePart =
			Options.NamePrefix +
			(FileExists(srcName) ?
				ChangeFileExt(ExtractFileName(srcName), AnsiString()) :
				ExtractFileName(srcName)) +
			Options.NameSuffix;

		// Get index part
		AnsiString indexPart;
		if(index != Options.FirstIndex || Options.AlwaysAddIndexPart) {
			indexPart = Options.IndexPrefix + index + Options.IndexSuffix;
		}

		// Get the full name
		AnsiString fullName;
		switch(Options.Layout) {
		case TDirCreationOptions::LAYOUT_NAME_INDEX:
			fullName = path + (namePart + indexPart).Trim();
			break;
		case TDirCreationOptions::LAYOUT_INDEX_NAME:
			fullName = path + (indexPart + namePart).Trim();
			break;
		default:
			throw EDirCreationFail(EXCEPTION_UNSUPPORTED_CONDITION);
		}
		if(!FileOrDirExists(fullName)) {
			return fullName;
		}
	}
	throw EDirCreationFail(AnsiString().sprintf(
		EXCEPTION_CANNOT_MAKE_DIR_NAME, srcName.c_str()));
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDirCreator::GetNextSrcName()
{
	AnsiString srcName = FullyQualifyFileName(SrcNames->Strings[Done]);
	if(srcName.Length() == 0) {
		throw EDirCreationFail(AnsiString().sprintf(
			EXCEPTION_INVALID_SRC_NAME, srcName.c_str()));
	}
	return srcName;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDirCreator::GetNextPath(AnsiString ASrcName)
{
	AnsiString path;
	switch(Options.Location) {
	case TDirCreationOptions::LOCATION_DESKTOP:
	case TDirCreationOptions::LOCATION_CUSTOM:
	case TDirCreationOptions::LOCATION_ASK:
		if(Done == 0) {
			EnsureFixedPath();
		}
		path = FixedPath;
		break;
	case TDirCreationOptions::LOCATION_SOURCE:
		path = ExtractFilePath(ASrcName);
		break;
	default:
		throw EDirCreationFail(EXCEPTION_UNSUPPORTED_CONDITION);
	}
	return path;
}
//---------------------------------------------------------------------------
void __fastcall TDirCreator::EnsureFixedPath()
{
	AnsiString location;
	switch(Options.Location) {
	case TDirCreationOptions::LOCATION_DESKTOP:
		location = GetSpecialDir(CSIDL_DESKTOPDIRECTORY);
		break;
	case TDirCreationOptions::LOCATION_CUSTOM:
		location = Options.CustomLocation;
		break;
	case TDirCreationOptions::LOCATION_ASK:
		location = DoAskLocation();
		break;
	default:
		throw EDirCreationFail(EXCEPTION_UNSUPPORTED_CONDITION);
	}
	if(!DirectoryExists(location)) {
		throw EDirCreationFail(EXCEPTION_CANNOT_GET_LOCATION);
	}
	FixedPath = IncludeTrailingPathDelimiter(location);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDirCreator::DoAskLocation()
{
	AnsiString location;
	if(OnAskLocation != NULL) {
		OnAskLocation(this, location);
	}
	return location;
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TDirCreator::TDirCreator(TStrings* ASrcNames,
		TDirCreationOptions AOptions) :
	TObject(),
	FSrcNames(NULL),
	FFixedPath(),
	FDone(0),
	FOptions(AOptions),
	FOnAskLocation(NULL)
{
	Initialize(ASrcNames);
}
//---------------------------------------------------------------------------
__fastcall TDirCreator::~TDirCreator()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
bool __fastcall TDirCreator::HasNext() const
{
	return Done < SrcNames->Count;
}
//---------------------------------------------------------------------------
void __fastcall TDirCreator::CreateNext()
{
	CreateNextDir();
	Done = Done + 1;
}
//---------------------------------------------------------------------------

