//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <clipbrd.hpp>
#include <filectrl.hpp>
#include <utilcls.h>
#include "TMainForm.h"
#include "TDirCreator.h"
#include "TFileExecutor.h"
#include "TFileNameList.h"
#include "TMessageDialog.h"
#include "SystemUtils.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm* MainForm = NULL;
//---------------------------------------------------------------------------
namespace
{
const char* CAPTION_BUTTON_OK =
	"OK";
const char* CAPTION_DIALOG_ABOUT =
	"About";
const char* PROMPT_OPEN_HELP =
	"The program opens the help document \"%s\".";
const char* PROMPT_SELECT_LOCATION =
	"Select a location to create directories.";
} // End of namespace
//---------------------------------------------------------------------------
// class TMainForm : public TForm
//---------------------------------------------------------------------------
// __published:
//---------------------------------------------------------------------------
void __fastcall TMainForm::ApplicationException(TObject* ASender,
	Exception* AException)
{
	ShowError(AException->Message);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ApplicationIdle(TObject* ASender, bool& ADone)
{
	try {
		try {
			ADone = true;
			Application->OnIdle = NULL;

			ProcessCommandParams();
		}
		catch(EAbort& e) {
			// Do nothing
		}
		catch(Exception& e) {
			ShowError(e.Message);
		}
		catch(...) {
			// Catch any exception to surely close this invisible form
			ShowError("An error occured.");
		}
	}
	__finally {
		Close();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DirCreatorAskLocation(TObject* ASender,
	AnsiString& ALocation)
{
	AnsiString location = DirData->LastLocation;
	SetForegroundWindow(Application->Handle);
	if(!SelectDirectory(PROMPT_SELECT_LOCATION, WideString(), location)) {
		Abort();
	}
	DirData->LastLocation = location;
	ALocation = location;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCloseQuery(TObject* ASender, bool& ACanClose)
{
	SaveSettings();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OptionsDialogApply(TObject* ASender,
	TOptionsDialogData AData)
{
	DirData->DirCreationOptions = AData.DirCreationOptions;
	SaveSettings();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OptionsDialogCommandAbout(TObject* ASender)
{
	TMessageDialog messageDialog;
	messageDialog.Caption = CAPTION_DIALOG_ABOUT;
	messageDialog.Message = INFO_ABOUT;
	messageDialog.Buttons->CommaText = CAPTION_BUTTON_OK;
	messageDialog.Default = 0;
	messageDialog.Cancel = 0;
	messageDialog.Type = TMessageDialog::TYPE_ORIGINAL;
	messageDialog.Icon->Assign(Application->Icon);
	messageDialog.Execute();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OptionsDialogDefault(TObject* ASender,
	TOptionsDialogData& AData)
{
	AData.DirCreationOptions = TDirData::GetDefDirCreationOptions();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OptionsDialogCommandHelp(TObject* ASender)
{
	if(Application->MessageBox(
			AnsiString().sprintf(PROMPT_OPEN_HELP,
				FILE_HELP.c_str()).c_str(),
			Application->Title.c_str(),
			MB_OKCANCEL | MB_ICONEXCLAMATION | MB_SETFOREGROUND) != IDOK) {
		Abort();
	}
	TFileExecutor fileExecutor;
	fileExecutor.Parent = Application->Handle;
	fileExecutor.OpenFile(FILE_HELP, AnsiString());
}
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
AnsiString TMainForm::INFO_ABOUT =
	"Doppeldirectory for Windows Version 1.1.0.\r\n\r\n"
	"Copyright (C) 2006-2007 Moochos. All rights reserved.";
AnsiString TMainForm::FILE_PATH =
	ExtractFilePath(FullyQualifyFileName(Application->ExeName));
AnsiString TMainForm::FILE_SETTINGS = FILE_PATH + "Settings.ini";
AnsiString TMainForm::FILE_HELP = FILE_PATH + "Help\\index.html";
//---------------------------------------------------------------------------
void __fastcall TMainForm::Initialize()
{
	try {
		Application->OnException = ApplicationException;
		Application->OnIdle = ApplicationIdle;
		FDirData = new TDirData();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Uninitialize()
{
	delete FDirData;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ProcessCommandParams()
{
	if(ParamCount() == 0) {
		ShowOptionsDialog();
	}
	else {
		CreateDirectories();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ShowOptionsDialog()
{
	TPtr<TOptionsDialog> optionsDialog = new TOptionsDialog(this);
	optionsDialog->OnApply = OptionsDialogApply;
	optionsDialog->OnCommandHelp = OptionsDialogCommandHelp;
	optionsDialog->OnCommandAbout = OptionsDialogCommandAbout;
	optionsDialog->OnDefault = OptionsDialogDefault;
	TOptionsDialogData optionsDialogData;
	optionsDialogData.DirCreationOptions = DirData->DirCreationOptions;
	if(optionsDialog->Execute(optionsDialogData)) {
		optionsDialog->OnApply(optionsDialog, optionsDialogData);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateDirectories()
{
	TPtr<TFileNameList> srcFiles = new TFileNameList();
	GetSrcFiles(srcFiles);
	TPtr<TDirCreator> dirCreator = new TDirCreator(srcFiles,
		DirData->DirCreationOptions);
	dirCreator->OnAskLocation = DirCreatorAskLocation;
	while(dirCreator->HasNext()) {
		dirCreator->CreateNext();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GetSrcFiles(TStrings* ASrcFiles)
{
	TPtr<TFileNameList> srcFiles = new TFileNameList();
	for(int i = 0; i < ParamCount(); i++) {
		srcFiles->Add(FullyQualifyFileName(ParamStr(i + 1)));
	}
	srcFiles->Sort();
	ASrcFiles->Assign(srcFiles);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ShowError(AnsiString AMessage)
{
	Application->MessageBox(
		AMessage.c_str(),
		Application->Title.c_str(),
		MB_OK | MB_ICONERROR | MB_SETFOREGROUND);
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* AOwner) :
	TForm(AOwner),
	FDirData(NULL)
{
	Initialize();
	try {
		LoadSettings();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
	Uninitialize();
}
//---------------------------------------------------------------------------

