//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <utilcls.h>
#include "TMainForm.h"
#include "TDirCreationOptions.h"
#include "TOptionsDialog.h"
#include "TQuoteIniFile.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace
{
const char* INI_MAINFORM = "MainForm";
const char* INI_MAINFORM_LASTLOCATION = "LastLocation";
const char* INI_DIRCREATIONOPTIONS = "DirCreationOptions";
const char* INI_DIRCREATIONOPTIONS_LOCATION = "Location";
const char* INI_DIRCREATIONOPTIONS_CUSTOMLOCATION = "CustomLocation";
const char* INI_DIRCREATIONOPTIONS_LAYOUT = "Layout";
const char* INI_DIRCREATIONOPTIONS_NAMEPREFIX = "NamePrefix";
const char* INI_DIRCREATIONOPTIONS_NAMESUFFIX = "NameSuffix";
const char* INI_DIRCREATIONOPTIONS_INDEXPREFIX = "IndexPrefix";
const char* INI_DIRCREATIONOPTIONS_INDEXSUFFIX = "IndexSuffix";
const char* INI_DIRCREATIONOPTIONS_FIRSTINDEX = "FirstIndex";
const char* INI_DIRCREATIONOPTIONS_LASTINDEX = "LastIndex";
const char* INI_DIRCREATIONOPTIONS_CREATELOCATION = "CreateLocation";
const char* INI_DIRCREATIONOPTIONS_ALWAYSADDINDEXPART = "AlwaysAddIndexPart";
} // End of namespace
//---------------------------------------------------------------------------
void __fastcall TMainForm::LoadSettings()
{
	for(;;) {
		if(!FileExists(FILE_SETTINGS)) {
			break;
		}
		TPtr<TQuoteIniFile> iniFile = new TQuoteIniFile(FILE_SETTINGS);
		{
			DirData->LastLocation = iniFile->ReadQuotedString(
				INI_MAINFORM,
				INI_MAINFORM_LASTLOCATION,
				DirData->LastLocation);
		}
		{
			TDirCreationOptions dirCreationOptions =
				DirData->DirCreationOptions;
			dirCreationOptions.Location = iniFile->ReadInteger(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_LOCATION,
				dirCreationOptions.Location);
			dirCreationOptions.CustomLocation = iniFile->ReadQuotedString(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_CUSTOMLOCATION,
				dirCreationOptions.CustomLocation);
			dirCreationOptions.Layout = iniFile->ReadInteger(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_LAYOUT,
				dirCreationOptions.Layout);
			dirCreationOptions.NamePrefix = iniFile->ReadQuotedString(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_NAMEPREFIX,
				dirCreationOptions.NamePrefix);
			dirCreationOptions.NameSuffix = iniFile->ReadQuotedString(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_NAMESUFFIX,
				dirCreationOptions.NameSuffix);
			dirCreationOptions.IndexPrefix = iniFile->ReadQuotedString(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_INDEXPREFIX,
				dirCreationOptions.IndexPrefix);
			dirCreationOptions.IndexSuffix = iniFile->ReadQuotedString(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_INDEXSUFFIX,
				dirCreationOptions.IndexSuffix);
			dirCreationOptions.FirstIndex = iniFile->ReadInteger(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_FIRSTINDEX,
				dirCreationOptions.FirstIndex);
			dirCreationOptions.LastIndex = iniFile->ReadInteger(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_LASTINDEX,
				dirCreationOptions.LastIndex);
			dirCreationOptions.CreateLocation = iniFile->ReadBool(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_CREATELOCATION,
				dirCreationOptions.CreateLocation);
			dirCreationOptions.AlwaysAddIndexPart = iniFile->ReadBool(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_ALWAYSADDINDEXPART,
				dirCreationOptions.AlwaysAddIndexPart);
			DirData->DirCreationOptions = dirCreationOptions;
		}
		DirData->Modified = false;
	 	break;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SaveSettings()
{
	for(;;) {
		if(!DirData->Modified) {
			break;
		}
		TPtr<TQuoteIniFile> iniFile = new TQuoteIniFile(FILE_SETTINGS);
		{
			iniFile->WriteQuotedString(
				INI_MAINFORM,
				INI_MAINFORM_LASTLOCATION,
				DirData->LastLocation);
		}
		{
			TDirCreationOptions dirCreationOptions =
				DirData->DirCreationOptions;
			iniFile->WriteInteger(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_LOCATION,
				dirCreationOptions.Location);
			iniFile->WriteQuotedString(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_CUSTOMLOCATION,
				dirCreationOptions.CustomLocation);
			iniFile->WriteInteger(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_LAYOUT,
				dirCreationOptions.Layout);
			iniFile->WriteQuotedString(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_NAMEPREFIX,
				dirCreationOptions.NamePrefix);
			iniFile->WriteQuotedString(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_NAMESUFFIX,
				dirCreationOptions.NameSuffix);
			iniFile->WriteQuotedString(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_INDEXPREFIX,
				dirCreationOptions.IndexPrefix);
			iniFile->WriteQuotedString(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_INDEXSUFFIX,
				dirCreationOptions.IndexSuffix);
			iniFile->WriteInteger(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_FIRSTINDEX,
				dirCreationOptions.FirstIndex);
			iniFile->WriteInteger(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_LASTINDEX,
				dirCreationOptions.LastIndex);
			iniFile->WriteBool(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_CREATELOCATION,
				dirCreationOptions.CreateLocation);
			iniFile->WriteBool(
				INI_DIRCREATIONOPTIONS,
				INI_DIRCREATIONOPTIONS_ALWAYSADDINDEXPART,
				dirCreationOptions.AlwaysAddIndexPart);
		}
		iniFile->UpdateFile();
		DirData->Modified = false;
		break;
	}
}
//---------------------------------------------------------------------------

