//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <FileCtrl.hpp>
#include "TOptionsDialog.h"
#include "SystemUtils.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TOptionsDialog* OptionsDialog = NULL;
//---------------------------------------------------------------------------
namespace
{
const char* EXCEPTION_INVALID_FILE_NAME =
	"Specify a valid filename.";
const char* EXCEPTION_INVALID_LOCATION =
	"Specify a valid location.";
const char* EXCEPTION_INVALID_RANGE =
	"Specify a valid range.";
const char* EXCEPTION_NO_ITEM_SELECTED =
	"Select an item.";
const char* EXCEPTION_NOT_INTEGER =
	"Specify an integer.";
const char* PROMPT_SELECT_LOCATION =
	"Select a location to create directories.";
} // End of namespace
//---------------------------------------------------------------------------
// class TOptionsDialog : public TForm
//---------------------------------------------------------------------------
// __published:
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::Button13Click(TObject* ASender)
{
	Data = EntriesToData();
	Modified = false;
	DoApply();
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::Button14Click(TObject* ASender)
{
	DataToEntries(DoDefault());
	Modified = true;
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::Button15Click(TObject* ASender)
{
	DoCommandHelp();
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::Button16Click(TObject* ASender)
{
	DoCommandAbout();
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::Button21Click(TObject* ASender)
{
	AnsiString location = Edit21->Text;
	if(!SelectDirectory(PROMPT_SELECT_LOCATION, WideString(), location)) {
		Abort();
	}
	Edit21->Text = location;
	Edit21->OnChange(Edit21);
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::CheckBoxClick(TObject* ASender)
{
	Modified = true;
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::ComboBoxChange(TObject* ASender)
{
	Modified = true;
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::Edit22Exit(TObject* ASender)
{
	EntriesToDirCreationOptions(false);
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::Edit23Exit(TObject* ASender)
{
	EntriesToDirCreationOptions(false);
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::Edit24Exit(TObject* ASender)
{
	EntriesToDirCreationOptions(false);
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::Edit25Exit(TObject* ASender)
{
	EntriesToDirCreationOptions(false);
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::Edit26Exit(TObject* ASender)
{
	EntriesToDirCreationOptions(false);
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::Edit27Exit(TObject* ASender)
{
	EntriesToDirCreationOptions(false);
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::EditChange(TObject* ASender)
{
	Modified = true;
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::FormShow(TObject* ASender)
{
	ShowWindow(Application->Handle, SW_HIDE);
	DataToEntries(Data);
	Modified = false;
	Change();
	SelectFirst();
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::FormCloseQuery(TObject* ASender,
	bool& ACanClose)
{
	if(ModalResult == Button11->ModalResult) {
		Data = EntriesToData();
	}
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::RadioButtonClick(TObject* ASender)
{
	Modified = true;
	Change();
}
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
__fastcall TOptionsDialog::EInvalidEntry::EInvalidEntry(
		AnsiString AMessage) :
	Exception(AMessage)
{
	// Do nothing
}
//---------------------------------------------------------------------------
TOptionsDialogData __fastcall TOptionsDialog::EntriesToData()
{
	TOptionsDialogData data;
	data.DirCreationOptions = EntriesToDirCreationOptions(true);
	return data;
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::DataToEntries(TOptionsDialogData AData)
{
	DirCreationOptionsToEntries(AData.DirCreationOptions);
	Change();
	SelectFirst();
}
//---------------------------------------------------------------------------
TDirCreationOptions __fastcall TOptionsDialog::EntriesToDirCreationOptions(
	bool AStrict)
{
	TDirCreationOptions dirCreationOptions;
	if(RadioButton21->Checked) {
		dirCreationOptions.Location = TDirCreationOptions::LOCATION_DESKTOP;
	}
	else if(RadioButton23->Checked) {
		dirCreationOptions.Location = TDirCreationOptions::LOCATION_SOURCE;
	}
	else if(RadioButton22->Checked) {
		dirCreationOptions.Location = TDirCreationOptions::LOCATION_CUSTOM;
	}
	else if(RadioButton24->Checked) {
		dirCreationOptions.Location = TDirCreationOptions::LOCATION_ASK;
	}
	else {
		RaiseInvalidEntry(EXCEPTION_NO_ITEM_SELECTED, RadioButton21);
	}
	dirCreationOptions.CustomLocation = Edit21->Text;
	if(ComboBox21->ItemIndex == 0) {
		dirCreationOptions.Layout = TDirCreationOptions::LAYOUT_NAME_INDEX;
	}
	else if(ComboBox21->ItemIndex == 1) {
		dirCreationOptions.Layout = TDirCreationOptions::LAYOUT_INDEX_NAME;
	}
	else {
		RaiseInvalidEntry(EXCEPTION_NO_ITEM_SELECTED, ComboBox21);
	}
	dirCreationOptions.NamePrefix = EditToFileName(Edit22);
	dirCreationOptions.NameSuffix = EditToFileName(Edit23);
	dirCreationOptions.IndexPrefix = EditToFileName(Edit24);
	dirCreationOptions.IndexSuffix = EditToFileName(Edit25);
	dirCreationOptions.FirstIndex = EditToInt(Edit26);
	dirCreationOptions.LastIndex = EditToInt(Edit27);
	dirCreationOptions.CreateLocation = CheckBox21->Checked;
	dirCreationOptions.AlwaysAddIndexPart = CheckBox22->Checked;
	if(AStrict) {
		if(dirCreationOptions.Location ==
					TDirCreationOptions::LOCATION_CUSTOM &&
				!IsAbsoluteFileName(dirCreationOptions.CustomLocation)) {
			RaiseInvalidEntry(EXCEPTION_INVALID_LOCATION, Edit21);
		}
		if(dirCreationOptions.FirstIndex > dirCreationOptions.LastIndex) {
			RaiseInvalidEntry(EXCEPTION_INVALID_RANGE, Edit26);
		}
	}
	return dirCreationOptions;
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::DirCreationOptionsToEntries(
	TDirCreationOptions ADirCreationOptions)
{
	if(ADirCreationOptions.Location ==
			TDirCreationOptions::LOCATION_DESKTOP) {
		RadioButton21->Checked = true;
	}
	else if(ADirCreationOptions.Location ==
			TDirCreationOptions::LOCATION_CUSTOM) {
		RadioButton22->Checked = true;
	}
	else if(ADirCreationOptions.Location ==
			TDirCreationOptions::LOCATION_SOURCE) {
		RadioButton23->Checked = true;
	}
	else if(ADirCreationOptions.Location ==
			TDirCreationOptions::LOCATION_ASK) {
		RadioButton24->Checked = true;
	}
	else {
		RadioButton21->Checked = false;
		RadioButton22->Checked = false;
		RadioButton23->Checked = false;
		RadioButton24->Checked = false;
	}
	Edit21->Text = ADirCreationOptions.CustomLocation;
	if(ADirCreationOptions.Layout ==
			TDirCreationOptions::LAYOUT_NAME_INDEX) {
		ComboBox21->ItemIndex = 0;
	}
	else if(ADirCreationOptions.Layout ==
			TDirCreationOptions::LAYOUT_INDEX_NAME) {
		ComboBox21->ItemIndex = 1;
	}
	else {
		ComboBox21->ItemIndex = -1;
	}
	Edit22->Text = ADirCreationOptions.NamePrefix;
	Edit23->Text = ADirCreationOptions.NameSuffix;
	Edit24->Text = ADirCreationOptions.IndexPrefix;
	Edit25->Text = ADirCreationOptions.IndexSuffix;
	Edit26->Text = ADirCreationOptions.FirstIndex;
	Edit27->Text = ADirCreationOptions.LastIndex;
	CheckBox21->Checked = ADirCreationOptions.CreateLocation;
	CheckBox22->Checked = ADirCreationOptions.AlwaysAddIndexPart;
}
//---------------------------------------------------------------------------
int __fastcall TOptionsDialog::EditToInt(TEdit* AEdit)
{
	int intValue;
	if(!TryStrToInt(AEdit->Text, intValue)) {
		RaiseInvalidEntry(EXCEPTION_NOT_INTEGER, AEdit);
	}
	return intValue;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TOptionsDialog::EditToFileName(TEdit* AEdit)
{
	AnsiString fileName = AEdit->Text;
	if(!IsValidFileName(fileName)) {
		RaiseInvalidEntry(EXCEPTION_INVALID_FILE_NAME, AEdit);
	}
	return fileName;
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::SetModified(bool AModified)
{
	if(AModified != FModified) {
		FModified = AModified;
		Change();
	}
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::SetOnApply(TApplyEvent AOnApply)
{
	if(AOnApply != FOnApply) {
		FOnApply = AOnApply;
		Change();
	}
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::SetOnDefault(TDefaultEvent AOnDefault)
{
	if(AOnDefault != FOnDefault) {
		FOnDefault = AOnDefault;
		Change();
	}
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::SetOnCommandHelp(TNotifyEvent AOnCommandHelp)
{
	if(AOnCommandHelp != FOnCommandHelp) {
		FOnCommandHelp = AOnCommandHelp;
		Change();
	}
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::SetOnCommandAbout(
	TNotifyEvent AOnCommandAbout)
{
	if(AOnCommandAbout != FOnCommandAbout) {
		FOnCommandAbout = AOnCommandAbout;
		Change();
	}
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::Change()
{
	Button13->Enabled = OnApply != NULL && Modified;
	Button14->Enabled = OnDefault != NULL;
	Button15->Enabled = OnCommandHelp != NULL;
	Button16->Enabled = OnCommandAbout != NULL;
	Edit21->Enabled = RadioButton22->Checked;
	Button21->Enabled = RadioButton22->Checked;
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::DoApply()
{
	if(OnApply != NULL) {
		OnApply(this, Data);
	}
}
//---------------------------------------------------------------------------
TOptionsDialogData __fastcall TOptionsDialog::DoDefault()
{
	TOptionsDialogData data = Data;
	if(OnDefault != NULL) {
		OnDefault(this, data);
	}
	return data;
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::DoCommandHelp()
{
	if(OnCommandHelp != NULL) {
		OnCommandHelp(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::DoCommandAbout()
{
	if(OnCommandAbout != NULL) {
		OnCommandAbout(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TOptionsDialog::RaiseInvalidEntry(AnsiString AMessage,
	TWinControl* AControl)
{
	ActiveControl = (AControl != NULL && AControl->CanFocus()) ?
		AControl :
		NULL;
	throw EInvalidEntry(AMessage);
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TOptionsDialog::TOptionsDialog(TComponent* AOwner) :
	TForm(AOwner),
	FData(),
	FModified(false),
	FOnApply(NULL),
	FOnDefault(NULL),
	FOnCommandHelp(NULL),
	FOnCommandAbout(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TOptionsDialog::~TOptionsDialog()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TOptionsDialog::Execute(TOptionsDialogData& AData)
{
	Data = AData;
	if(ShowModal() == Button11->ModalResult) {
		AData = Data;
		return true;
	}
	else {
		return false;
	}
}
//---------------------------------------------------------------------------

