//---------------------------------------------------------------------------
// TMessageDialog Class
// Copyright (C) 2006-2007 Moochos. All rights reserved.
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <StrUtils.hpp>
#include "TMessageDialog.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// class TMessageDialog
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
// class TMessageDialog::EMessageDialogError : public Exception
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TMessageDialog::EMessageDialogError::EMessageDialogError(
	AnsiString AMessage) : Exception(AMessage)
{
	// Do nothing
}
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall TMessageDialog::GetControls(TCustomForm* AForm,
	TList* AControls)
{
	TStack* rest = NULL;
	try {
		AControls->Clear();
		rest = new TStack();
		rest->Push(AForm);
		TControl* control;
		TWinControl* parent;
		while(rest->Count() > 0) {
			control = static_cast<TControl*>(rest->Pop());
			AControls->Add(control);
			if(control->InheritsFrom(__classid(TWinControl))) {
				parent = dynamic_cast<TWinControl*>(control);
				for(int i = 0; i < parent->ControlCount; i++) {
					rest->Push(parent->Controls[i]);
				}
			}
		}
	}
	__finally {
		delete rest;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMessageDialog::EnableCloseButton(TCustomForm* AForm,
	bool AEnabled)
{
	EnableMenuItem(
		GetSystemMenu(AForm->Handle, false),
		SC_CLOSE,
		MF_BYCOMMAND | (AEnabled ? MF_ENABLED : MF_GRAYED));
}
//---------------------------------------------------------------------------
void __fastcall TMessageDialog::Initialize()
{
	try {
		FButtons = new TStringList();
		FIcon = new TIcon();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMessageDialog::Uninitialize()
{
	delete FIcon;
	delete FButtons;
}
//---------------------------------------------------------------------------
TForm* __fastcall TMessageDialog::MakeDialog()
{
	TForm* dialog = NULL;
	try {
		dialog = CreateMessageDialog(GetDialogMessage(), GetMsgDlgType(),
			GetMsgDlgButtons());
		dialog->Caption = Caption;
		dialog->OnKeyDown = NULL; // Disable "Ctrl+C"
		EnableCloseButton(dialog, CanCancel());
		if(!CanCancel()) {
			AddCancelStopper(dialog);
		}
		ModifyControls(dialog);
	}
	catch(...) {
		delete dialog;
		throw;
	}
	return dialog;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TMessageDialog::GetDialogMessage()
{
	// Offset the TLabel::ShowAccelChar property
	return AnsiReplaceStr(Message, "&", "&&");
}
//---------------------------------------------------------------------------
TMsgDlgType __fastcall TMessageDialog::GetMsgDlgType()
{
	TMsgDlgType msgDlgType;
	switch(Type) {
		case TYPE_WARNING: msgDlgType = mtWarning; break;
		case TYPE_ERROR: msgDlgType = mtError; break;
		case TYPE_INFORMATION: msgDlgType = mtInformation; break;
		case TYPE_QUESTION: msgDlgType = mtConfirmation; break;
		case TYPE_ORIGINAL: msgDlgType = mtError; break;
		case TYPE_NONE: msgDlgType = mtCustom; break;
		default: throw EMessageDialogError("Invalid dialog type");
	}
	return msgDlgType;
}
//---------------------------------------------------------------------------
TMsgDlgButtons __fastcall TMessageDialog::GetMsgDlgButtons()
{
	TMsgDlgButtons msgDlgButtons;
	const int buttons[] = {
		mbYes,
		mbNo,
		mbOK,
		//mbCancel,
		mbAbort,
		mbRetry,
		mbIgnore,
		mbAll,
		mbNoToAll,
		mbYesToAll,
		//mbHelp,
	};
	if(Buttons->Count > sizeof(buttons) / sizeof(int)) {
		throw EMessageDialogError("Too many dialog buttons");
	}
	for(int i = 0; i < Buttons->Count; i++) {
		msgDlgButtons << static_cast<TMsgDlgBtn>(buttons[i]);
	}
	return msgDlgButtons;
}
//---------------------------------------------------------------------------
bool __fastcall TMessageDialog::CanCancel()
{
	return (FCancel >= 0 && FCancel <= Buttons->Count - 1) ||
		Buttons->Count <= 0;
}
//---------------------------------------------------------------------------
void __fastcall TMessageDialog::AddCancelStopper(TForm* ADialog)
{
	// Disable "Alt+F4"
	class TCancelStopper : public TComponent
	{
	public:
		__fastcall TCancelStopper(TForm* ADialog) :
			TComponent(ADialog)
		{
			ADialog->OnCloseQuery = DialogCloseQuery;
		}
		void __fastcall DialogCloseQuery(TObject* ASender, bool& ACanClose)
		{
			if(dynamic_cast<TForm*>(ASender)->ModalResult == mrCancel) {
				ACanClose = false;
			}
		}
	};
	new TCancelStopper(ADialog);
}
//---------------------------------------------------------------------------
void __fastcall TMessageDialog::ModifyControls(TForm* ADialog)
{
	TList* controls = NULL;
	try {
		controls = new TList();
		GetControls(ADialog, controls);
		TControl* control;
		TButton* button;
		TImage* image;
		for(int i = 0; i < controls->Count; i++) {
			control = static_cast<TControl*>(controls->Items[i]);
			if(control->InheritsFrom(__classid(TButton))) {
				button = dynamic_cast<TButton*>(control);
				button->Caption = Buttons->Strings[button->TabOrder];
				button->Default = button->TabOrder == Default;
				if(button->Default) {
					ADialog->ActiveControl = button;
				}
				button->Cancel = button->TabOrder == Cancel;
				if(button->Cancel) {
					button->ModalResult = mrCancel;
				}
			}
			else if(control->InheritsFrom(__classid(TImage))) {
				if(Type == TYPE_ORIGINAL) {
					image = dynamic_cast<TImage*>(control);
					image->Picture->Icon->Assign(Icon);
				}
			}
		}
	}
	__finally {
		delete controls;
	}
}
//---------------------------------------------------------------------------
int __fastcall TMessageDialog::ShowDialog(TForm* ADialog)
{
	int result = -1;
	TModalResult modalResult = ADialog->ShowModal();
	TList* controls = NULL;
	try {
		controls = new TList();
		GetControls(ADialog, controls);
		TControl* control;
		TButton* button;
		for(int i = 0; i < controls->Count; i++) {
			control = static_cast<TControl*>(controls->Items[i]);
			if(control->InheritsFrom(__classid(TButton))) {
				button = dynamic_cast<TButton*>(control);
				if(button->ModalResult == modalResult) {
					result = button->TabOrder;
					break;
				}
			}
		}
	}
	__finally {
		delete controls;
	}
	return result;
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TMessageDialog::TMessageDialog() :
	FCaption(),
	FMessage(),
	FButtons(NULL),
	FDefault(-1),
	FCancel(-1),
	FType(TYPE_NONE),
	FIcon(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TMessageDialog::~TMessageDialog()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
int __fastcall TMessageDialog::Execute()
{
	int result;
	TForm* dialog = NULL;
	try {
		dialog = MakeDialog();
		result = ShowDialog(dialog);
	}
	__finally {
		delete dialog;
	}
	return result;
}
//---------------------------------------------------------------------------

